// this file is part of
// markonvert - Fortran unformatted record marker converter
//
// Copyright (C) 2005-2006  Francesco Poli
//
// This work is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 2,
// as published by the Free Software Foundation.
// 
// This work is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this work; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifndef MK_FROM64READER_H
#define MK_FROM64READER_H

#include "converter.h"

namespace mk
{
    // class for reading from a 64 bit marker file
    class From64Reader : public virtual Converter
      {
        protected:
            virtual void read_start_mark()
              {
                // read record start marker from stdin
                int32 zero;
                std::cin.read(reinterpret_cast<char*>(&record_len),
                              sizeof(int32));
                std::cin.read(reinterpret_cast<char*>(&zero),
                              sizeof(int32));
                Assert<Ex_long_record>(zero == 0);
                Assert<Ex_neg_length>(record_len >= 0);
              }

            virtual void read_end_mark()
              {
                // read record end marker from stdin
                int32 end, zero;
                std::cin.read(reinterpret_cast<char*>(&end),
                              sizeof(int32));
                std::cin.read(reinterpret_cast<char*>(&zero),
                              sizeof(int32));
                Assert<Ex_marks_diff>(end == record_len);
                Assert<Ex_marks_diff>(zero == 0);
              }
      };

    /* works for little endian data on little endian machines!
       TODO: the general case should be addressed... */
}

#endif // not MK_FROM64READER_H
