// this file is part of
// markonvert - Fortran unformatted record marker converter
//
// Copyright (C) 2005-2007  Francesco Poli
//
// This work is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 2,
// as published by the Free Software Foundation.
// 
// This work is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this work; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

// #include "config.h"
#include <string>
#include <memory>
#include "usage.h"
#include "realconverters.h"

int
main(int argc, char* argv[])
{
    if (argc != 2)
      {
        mk::usage(argv[0]);
        std::exit(1);
      }

    try
      {
        std::auto_ptr<mk::Converter> conv;
        
        // instantiate the appropriate converter
        std::string conversion(argv[1]);
        if      (conversion == "--from32to64")
          {
            conv.reset(new mk::From32To64);
          }
        else if (conversion == "--from64to32")
          {
            conv.reset(new mk::From64To32);
          }
        else
          {
                mk::usage(argv[0]);
                std::exit(2);
          }

        conv->go();  // perform the actual conversion
      }
    catch (mk::Ex_int_not_4)
      {
        std::cerr << "Error: int32 is not 4 byte long!"
                  << " cannot operate!"
                  << std::endl;
        std::exit(11);
      }
    catch (mk::Ex_chr_not_1)
      {
        std::cerr << "Error: char is not 1 byte long on this architecture!"
                  << " cannot operate!"
                  << std::endl;
        std::exit(12);
      }
    catch (mk::Ex_neg_length)
      {
        std::cerr << "Error: found a negative record length!"
                  << " cannot process it!"
                  << std::endl;
        std::exit(13);
      }
    catch (mk::Ex_long_record)
      {
        std::cerr << "Error: found a 2^31 byte (or greater) record length!"
                  << " cannot process it!"
                  << std::endl;
        std::exit(13);
      }
    catch (mk::Ex_marks_diff)
      {
        std::cerr << "Error: input inconsistency detected!"
                  << std::endl
                  << "Start and end markers disagree on record length!"
                  << " cannot proceed!"
                  << std::endl;
        std::exit(13);
      }
    catch (std::ios::failure)
      {
        std::cerr << "I/O error: found a malformed record!"
                  << std::endl;
        std::exit(14);
      }
    catch (...)
      {
        std::cerr << "Abort: something went wrong, and we don't even know what!"
                  << std::endl;
        std::exit(63);
      }

    return 0;
}
